@@ -1396,12 +1396,12 @@ typedef enum {
     MOKS_OFF,
     // Used when receiving the state and the level is two.
     MOKS_ENABLED,
-    // Used after outputting t_KE when the state was MOKS_ENABLED.  We do not
-    // really know if t_KE actually disabled the protocol, the following t_KI
+    // Used after outputting t_TE when the state was MOKS_ENABLED.  We do not
+    // really know if t_TE actually disabled the protocol, the following t_TI
     // is expected to request the state, but the response may come only later.
     MOKS_DISABLED,
-    // Used after outputting t_KE when the state was not MOKS_ENABLED.
-    MOKS_AFTER_T_KE,
+    // Used after outputting t_TE when the state was not MOKS_ENABLED.
+    MOKS_AFTER_T_TE,
 } mokstate_T;
 
 // Set when a response to XTQMODKEYS was received.  Only works for xterm
@@ -1416,12 +1416,12 @@ typedef enum {
     KKPS_OFF,
     // Used when receiving the state and the flags are non-zero.
     KKPS_ENABLED,
-    // Used after outputting t_KE when the state was KKPS_ENABLED.  We do not
-    // really know if t_KE actually disabled the protocol, the following t_KI
+    // Used after outputting t_TE when the state was KKPS_ENABLED.  We do not
+    // really know if t_TE actually disabled the protocol, the following t_TI
     // is expected to request the state, but the response may come only later.
     KKPS_DISABLED,
-    // Used after outputting t_KE when the state was not KKPS_ENABLED.
-    KKPS_AFTER_T_KE,
+    // Used after outputting t_TE when the state was not KKPS_ENABLED.
+    KKPS_AFTER_T_TE,
 } kkpstate_T;
 
 EXTERN kkpstate_T kitty_protocol_state INIT(= KKPS_INITIAL);
@@ -1905,9 +1905,6 @@ EXTERN char need_key_msg[]  INIT(= N_("Need encryption key for \"%s\""));
 EXTERN int xsmp_icefd INIT(= -1);   // The actual connection
 #endif
 
-// For undo we need to know the lowest time possible.
-EXTERN time_T starttime;
-
 #ifdef STARTUPTIME
 EXTERN FILE *time_fd INIT(= NULL);  // where to write startup timing
 #endif
@@ -2038,3 +2035,7 @@ EXTERN int skip_win_fix_cursor INIT(= FALSE);
 EXTERN int skip_win_fix_scroll INIT(= FALSE);
 // Skip update_topline() call while executing win_fix_scroll().
 EXTERN int skip_update_topline INIT(= FALSE);
+
+// 'showcmd' buffer shared between normal.c and statusline.c
+#define SHOWCMD_BUFLEN (SHOWCMD_COLS + 1 + 30)
+EXTERN char_u showcmd_buf[SHOWCMD_BUFLEN];
